@@ -15,13 +15,21 @@ Architecture:
 import asyncio
 import aiohttp
 
-from synqed.agent import Agent, AgentLogicContext, ResponseBuilder
+from synqed.agent import (
+    Agent, 
+    AgentLogicContext, 
+    ResponseBuilder,
+    parse_llm_response,
+    extract_partial_json_content,
+)
 from synqed.server import AgentServer
 from synqed.memory import AgentMemory, InboxMessage
 from synqed.router import MessageRouter
 from synqed.planner import PlannerLLM, TaskTreePlan, TaskTreeNode
 from synqed.display import MessageDisplay
 from synqed.execution_engine import WorkspaceExecutionEngine
+from synqed.workspace_manager import Workspace, WorkspaceManager, AgentRuntimeRegistry
+from synqed.a2a_remote_agent import RemoteA2AAgent
 
 __version__ = "2.0.0"
 
@@ -32,6 +40,9 @@ __all__ = [
     "ResponseBuilder",
     "AgentServer",
     
+    # === Remote A2A Integration ===
+    "RemoteA2AAgent",
+    
     # === Memory ===
     "AgentMemory",
     "InboxMessage",
@@ -39,6 +50,11 @@ __all__ = [
     # === Routing ===
     "MessageRouter",
     
+    # === Workspace Management ===
+    "Workspace",
+    "WorkspaceManager",
+    "AgentRuntimeRegistry",
+    
     # === Planning ===
     "PlannerLLM",
     "TaskTreePlan",
@@ -48,6 +64,10 @@ __all__ = [
     "WorkspaceExecutionEngine",
     "MessageDisplay",
     
+    # === Response Parsing Utils ===
+    "parse_llm_response",
+    "extract_partial_json_content",
+    
     # === Utils ===
     "asyncio",
     "aiohttp",
