@@ -6,11 +6,92 @@ with inbox memory access and structured JSON response handling.
 """
 
 import json
-from typing import Any, Callable, Coroutine, Optional
+from typing import Any, Callable, Coroutine, Optional, List
 
 from synqed.memory import AgentMemory, InboxMessage
 
 
+def parse_llm_response(response_text: str) -> str:
+    """
+    Parse LLM response by stripping markdown code blocks.
+    
+    Many LLMs wrap JSON responses in markdown code blocks like:
+    ```json
+    {"send_to": "Agent", "content": "..."}
+    ```
+    
+    This function removes those code blocks to extract the actual content.
+    
+    Args:
+        response_text: Raw LLM response text
+        
+    Returns:
+        Cleaned response text with markdown code blocks removed
+    """
+    if "```" not in response_text:
+        return response_text
+    
+    # Find the JSON content between code blocks
+    start_idx = response_text.find("```")
+    if start_idx != -1:
+        # Skip the opening ``` and optional language identifier
+        start_idx = response_text.find("\n", start_idx)
+        if start_idx != -1:
+            end_idx = response_text.find("```", start_idx)
+            if end_idx != -1:
+                return response_text[start_idx:end_idx].strip()
+    
+    # Fallback: return original
+    return response_text
+
+
+def extract_partial_json_content(response_text: str) -> Optional[dict]:
+    """
+    Extract content from partial/truncated JSON responses.
+    
+    Handles cases where LLM response was truncated mid-JSON, like:
+    {"send_to": "USER", "content": "story text...
+    
+    Args:
+        response_text: Potentially truncated JSON response
+        
+    Returns:
+        Extracted dict with send_to and content if parseable, None otherwise
+    """
+    # Check if response was INTENDED for specific recipient
+    send_to_match = None
+    for pattern in ['"send_to": "', '"send_to":"']:
+        if pattern in response_text:
+            start = response_text.find(pattern) + len(pattern)
+            end = response_text.find('"', start)
+            if end != -1:
+                send_to_match = response_text[start:end]
+                break
+    
+    if not send_to_match:
+        return None
+    
+    # Try to find the content field
+    content_match = None
+    for pattern in ['"content": "', '"content":"']:
+        if pattern in response_text:
+            content_start = response_text.find(pattern) + len(pattern)
+            # Extract until end or closing quote
+            content_text = response_text[content_start:]
+            # Try to find natural end (closing quote followed by })
+            # But handle truncation gracefully
+            content_match = content_text.rstrip('"}')
+            break
+    
+    if content_match is None:
+        return None
+    
+    return {
+        "send_to": send_to_match,
+        "content": content_match
+    }
+
+
 class ResponseBuilder:
     """
     Helper class for building structured responses.
@@ -95,21 +176,33 @@ class AgentLogicContext:
     - Latest incoming message
     - ResponseBuilder helper
     - send() helper method for convenience
+    - get_conversation_history() for formatted conversation history
+    - workspace reference for transcript access
     
     WARNING: Agent logic functions must NOT directly use any router.
     Routing is strictly handled by ExecutionEngine + Workspace.
     """
     
-    def __init__(self, memory: AgentMemory, default_target: Optional[str] = None):
+    def __init__(
+        self, 
+        memory: AgentMemory, 
+        default_target: Optional[str] = None,
+        workspace: Optional[Any] = None,
+        agent_name: Optional[str] = None
+    ):
         """
         Initialize the logic context.
         
         Args:
             memory: Agent's memory instance
             default_target: Default target agent if logic doesn't specify one
+            workspace: Optional workspace reference for transcript access
+            agent_name: Optional agent name for conversation history filtering
         """
         self.memory = memory
         self.default_target = default_target
+        self.workspace = workspace
+        self.agent_name = agent_name
         self._response_builder = ResponseBuilder()
     
     @property
@@ -147,6 +240,91 @@ class AgentLogicContext:
             Structured response dictionary
         """
         return ResponseBuilder().send_to(send_to).content(content).build()
+    
+    def get_conversation_history(
+        self, 
+        format: str = "text",
+        include_system_messages: bool = False,
+        parse_json_content: bool = True
+    ) -> str | List[dict]:
+        """
+        Get formatted conversation history for this agent.
+        
+        This method extracts the conversation history from the workspace transcript,
+        including both messages received and sent by this agent. It automatically
+        parses JSON content to extract the actual message content.
+        
+        Args:
+            format: Output format - "text" for formatted string, "raw" for list of dicts
+            include_system_messages: Whether to include system messages like [startup]
+            parse_json_content: Whether to parse JSON content and extract "content" field
+            
+        Returns:
+            Formatted conversation history as string (if format="text") or list of dicts (if format="raw")
+            
+        Example:
+            ```python
+            async def agent_logic(context):
+                # Get conversation history as formatted text
+                history = context.get_conversation_history()
+                
+                # Pass to LLM
+                response = await llm.chat(history)
+                return context.send("Editor", response)
+            ```
+        """
+        if not self.workspace or not self.agent_name:
+            # No workspace context, return empty
+            return "" if format == "text" else []
+        
+        transcript = self.workspace.router.get_transcript()
+        conversation_parts = []
+        raw_messages = []
+        
+        for entry in transcript:
+            sender = entry.get("from", "")
+            recipient = entry.get("to", "")
+            content = entry.get("content", "")
+            
+            # Skip system messages unless requested
+            if not include_system_messages:
+                if content == "[startup]" or content.startswith("[subteam_result]"):
+                    continue
+            
+            # Only include messages involving this agent (received or sent)
+            if recipient == self.agent_name or sender == self.agent_name:
+                # Parse JSON content if requested
+                display_content = content
+                if parse_json_content:
+                    try:
+                        parsed = json.loads(content)
+                        if isinstance(parsed, dict) and "content" in parsed:
+                            display_content = parsed["content"]
+                    except (json.JSONDecodeError, TypeError):
+                        # Not JSON or no "content" field, use as-is
+                        pass
+                
+                # Build message entry
+                if format == "text":
+                    if sender == "USER":
+                        conversation_parts.append(f"User: {display_content}")
+                    elif sender == self.agent_name:
+                        conversation_parts.append(f"{self.agent_name} (you): {display_content}")
+                    else:
+                        conversation_parts.append(f"{sender}: {display_content}")
+                else:
+                    raw_messages.append({
+                        "sender": sender,
+                        "recipient": recipient,
+                        "content": display_content,
+                        "original_content": content,
+                        "timestamp": entry.get("timestamp", "")
+                    })
+        
+        if format == "text":
+            return "\n\n".join(conversation_parts)
+        else:
+            return raw_messages
 
 
 class Agent:
@@ -297,6 +475,12 @@ class Agent:
         """
         Ensure the response is valid structured JSON with strict validation.
         
+        This method automatically handles:
+        - Markdown code block stripping (```json ... ```)
+        - Partial/truncated JSON responses
+        - Missing "send_to" fields (uses default_target)
+        - Missing "content" fields (converts whole object to JSON string)
+        
         Validation rules:
         - If missing "send_to" → use default_target
         - If missing "content" → convert whole object to JSON string
@@ -338,10 +522,14 @@ class Agent:
                 "content": content
             }
         
-        # If string, try to parse as JSON
+        # If string, try to parse as JSON (with automatic cleaning)
         if isinstance(result, str):
+            # Step 1: Strip markdown code blocks
+            cleaned_result = parse_llm_response(result)
+            
+            # Step 2: Try to parse as JSON
             try:
-                parsed = json.loads(result)
+                parsed = json.loads(cleaned_result)
                 if isinstance(parsed, dict):
                     send_to = parsed.get("send_to")
                     content = parsed.get("content")
@@ -375,7 +563,12 @@ class Agent:
                         "content": result
                     }
             except json.JSONDecodeError:
-                # Not JSON, wrap it
+                # Step 3: Try to extract partial JSON (handles truncated responses)
+                partial = extract_partial_json_content(cleaned_result)
+                if partial:
+                    return partial
+                
+                # Step 4: Not JSON, wrap it
                 if effective_default is None:
                     raise ValueError(
                         "Response is not valid JSON and no default_target is set. "
