@@ -47,7 +47,12 @@ class Context(AgentLogicContext):
         event_payload: Optional[dict] = None,
     ):
         """Initialize the context."""
-        super().__init__(memory=memory, default_target=default_target)
+        super().__init__(
+            memory=memory, 
+            default_target=default_target,
+            workspace=workspace,
+            agent_name=agent_name
+        )
         self.agent_name = agent_name
         self.workspace = workspace
         self.workspace_id = workspace_id
@@ -78,19 +83,19 @@ class WorkspaceExecutionEngine:
     - Complete transcript tracking
     """
     
-    # Global limits
-    max_total_workspaces = 50
-    max_workspace_depth = 8
-    max_subteam_requests_per_agent = 3
-    max_events_per_workspace = 2000
-    max_events_per_cycle = 50
-    fatal_cycle_threshold = 5  # Cycles with no output and non-empty queue
-    
     def __init__(
         self,
         planner: PlannerLLM,
         workspace_manager: WorkspaceManager,
         enable_display: bool = True,
+        max_cycles: int = 20,
+        max_events_per_cycle: int = 50,
+        max_events_per_workspace: int = 2000,
+        max_agent_turns: Optional[int] = None,
+        max_total_workspaces: int = 50,
+        max_workspace_depth: int = 8,
+        max_subteam_requests_per_agent: int = 3,
+        fatal_cycle_threshold: int = 5,
     ):
         """
         Initialize the workspace execution engine.
@@ -99,10 +104,30 @@ class WorkspaceExecutionEngine:
             planner: PlannerLLM instance for creating subteam subtrees
             workspace_manager: WorkspaceManager instance for workspace operations
             enable_display: Whether to enable real-time message display (default: True)
+            max_cycles: Maximum number of event processing cycles per workspace (default: 20)
+            max_events_per_cycle: Maximum events to process in a single cycle (default: 50)
+            max_events_per_workspace: Maximum total events per workspace (default: 2000)
+            max_agent_turns: Maximum number of agent responses/turns before stopping (default: None = unlimited)
+            max_total_workspaces: Maximum total workspaces that can be created (default: 50)
+            max_workspace_depth: Maximum nesting depth for workspaces (default: 8)
+            max_subteam_requests_per_agent: Maximum subteam requests per agent (default: 3)
+            fatal_cycle_threshold: Cycles with no output before killing workspace (default: 5)
         """
         self.planner = planner
         self.workspace_manager = workspace_manager
+        self._workspace_schedulers: dict[str, EventScheduler] = {}  # Track schedulers for all workspaces
         self._total_workspaces_created = 0
+        
+        # Configurable execution limits
+        self.max_cycles = max_cycles
+        self.max_events_per_cycle = max_events_per_cycle
+        self.max_events_per_workspace = max_events_per_workspace
+        self.max_agent_turns = max_agent_turns
+        self.max_total_workspaces = max_total_workspaces
+        self.max_workspace_depth = max_workspace_depth
+        self.max_subteam_requests_per_agent = max_subteam_requests_per_agent
+        self.fatal_cycle_threshold = fatal_cycle_threshold
+        
         # Global workspace execution queue for asynchronous hierarchical execution
         self.global_workspace_queue: asyncio.Queue[str] = asyncio.Queue()
         # Track which workspaces are currently executing to prevent re-entry
@@ -137,12 +162,12 @@ class WorkspaceExecutionEngine:
     
     async def run_global_scheduler(self) -> None:
         """
-        Process all workspaces in the global execution queue.
+        Process all workspaces in the global execution queue IN PARALLEL.
         
         This method continuously processes workspaces from the queue until
-        the queue is empty. Each workspace runs independently without recursion.
-        Workspaces may schedule child workspaces during execution, which will
-        be processed in subsequent iterations.
+        the queue is empty. Workspaces run concurrently using asyncio.gather,
+        allowing true parallel execution. Workspaces may schedule child workspaces
+        during execution, which will be processed in subsequent iterations.
         """
         # Continue processing until queue is empty
         # New workspaces may be scheduled during execution
@@ -153,17 +178,26 @@ class WorkspaceExecutionEngine:
             if self.global_workspace_queue.empty():
                 break
             
-            # Process all workspaces currently in queue
+            # Collect all workspaces currently in queue
             batch_size = self.global_workspace_queue.qsize()
+            workspace_ids = []
             for _ in range(batch_size):
                 try:
                     workspace_id = self.global_workspace_queue.get_nowait()
-                    await self.run_workspace(workspace_id=workspace_id, max_cycles=20)
-                    processed_count += 1
+                    workspace_ids.append(workspace_id)
                 except asyncio.QueueEmpty:
                     break
+            
+            if workspace_ids:
+                logger.debug(f"Running {len(workspace_ids)} workspaces in parallel: {workspace_ids}")
+                
+                # Run all workspaces in this batch CONCURRENTLY
+                tasks = [self.run_workspace(workspace_id=wid) for wid in workspace_ids]
+                try:
+                    await asyncio.gather(*tasks, return_exceptions=True)
+                    processed_count += len(workspace_ids)
                 except Exception as e:
-                    logger.error(f"Error executing workspace {workspace_id} from queue: {e}")
+                    logger.error(f"Error in parallel workspace execution: {e}")
     
     async def run(self, root_workspace_id: str) -> None:
         """
@@ -192,7 +226,7 @@ class WorkspaceExecutionEngine:
     async def run_workspace(
         self,
         workspace_id: str,
-        max_cycles: int = 20,
+        max_cycles: Optional[int] = None,
     ) -> None:
         """
         Execute a workspace's agents using event-driven scheduling.
@@ -203,8 +237,11 @@ class WorkspaceExecutionEngine:
         
         Args:
             workspace_id: ID of workspace to execute
-            max_cycles: Maximum number of event processing cycles to execute
+            max_cycles: Maximum number of event processing cycles (default: uses engine's max_cycles)
         """
+        # Use instance max_cycles if not provided
+        if max_cycles is None:
+            max_cycles = self.max_cycles
         workspace = self.workspace_manager.get_workspace(workspace_id)
         
         # Prevent re-entry
@@ -217,8 +254,10 @@ class WorkspaceExecutionEngine:
         self._running_workspaces.add(workspace_id)
         
         try:
-            # Create event scheduler for this workspace
-            scheduler = EventScheduler()
+            # Get or create shared event scheduler for this workspace
+            if workspace_id not in self._workspace_schedulers:
+                self._workspace_schedulers[workspace_id] = EventScheduler()
+            scheduler = self._workspace_schedulers[workspace_id]
         
             # Schedule startup events for all agents (only on first execution)
             if not workspace.has_started:
@@ -259,8 +298,10 @@ class WorkspaceExecutionEngine:
             cycle = 0
             consecutive_no_output_cycles = 0
             total_events_processed = 0
+            agent_turns = 0  # Count of actual agent responses
+            task_complete = False  # Flag to stop execution when USER receives a message
             
-            while scheduler.has_pending_events() and cycle < max_cycles:
+            while scheduler.has_pending_events() and cycle < max_cycles and not task_complete:
                 # Event loop protection: max events per workspace
                 if total_events_processed >= self.max_events_per_workspace:
                     if self.display:
@@ -360,13 +401,21 @@ class WorkspaceExecutionEngine:
                         # ExecutionEngine is the only component allowed to route messages;
                         # agent logic may not use routers. All routing must go through workspace.route_message.
                         # Agent.process enforces router isolation and structured return format.
-                        result = await agent.process(context)
                         
-                        # Mark message as processed after successful logic execution
-                        if message_id:
-                            agent.memory.mark_message_processed(message_id)
+                        # Check if this is a local agent or remote A2A agent
+                        if hasattr(agent, 'process'):
+                            # Local agent built with Synqed - call process() with context
+                            result = await agent.process(context)
+                            
+                            # Mark message as processed after successful logic execution
+                            if message_id:
+                                agent.memory.mark_message_processed(message_id)
+                        else:
+                            # Remote A2A agent - call get_response() (messages already buffered)
+                            result = await agent.get_response()
+                            # No memory to mark - remote agent manages its own state
                         
-                        # Hardened agent output validation
+                        # Hardened agent output validation (supports broadcast)
                         agent_response = self._validate_and_normalize_response(
                             result, agent_name, workspace
                         )
@@ -374,80 +423,147 @@ class WorkspaceExecutionEngine:
                         if agent_response is None:
                             continue  # Invalid response, skip
                         
-                        # Handle USER messages: route but do not schedule events
-                        if agent_response["send_to"] == "USER":
+                        # Handle BROADCAST: convert single dict to list for uniform processing
+                        responses_to_process = [agent_response] if isinstance(agent_response, dict) else agent_response
+                        
+                        # Process each response (single or broadcast)
+                        for single_response in responses_to_process:
+                            # Handle USER messages: route and STOP execution (task complete)
+                            if single_response["send_to"] == "USER":
+                                # Display the message
+                                if self.display:
+                                    self.display.display_message(
+                                        sender=agent_name,
+                                        recipient="USER",
+                                        content=single_response["content"]
+                                    )
+                                
+                                await workspace.route_message(
+                                    sender=agent_name,
+                                    recipient="USER",
+                                    content=single_response["content"],
+                                    manager=self.workspace_manager,
+                                )
+                                
+                                # Task complete - break out of ALL loops
+                                logger.info(f"Agent {agent_name} sent to USER - task complete, stopping workspace")
+                                task_complete = True
+                                # Clear remaining events
+                                while scheduler.has_pending_events():
+                                    scheduler.pop_next_event()
+                                break  # Exit event processing loop
+                            
+                            # Check for subteam request
+                            subteam_handled = await self._handle_subteam_request(
+                                workspace=workspace,
+                                workspace_id=workspace_id,
+                                agent_name=agent_name,
+                                response=single_response,
+                                scheduler=scheduler,  # Only used for error cases
+                            )
+                            
+                            if subteam_handled:
+                                cycle_had_output = True
+                                continue
+                            
+                            # Check for infinite self-message loops
+                            if self._is_self_message_loop(
+                                agent, single_response["send_to"], agent_name, single_response["content"]
+                            ):
+                                logger.warning(
+                                    f"Blocked self-message loop for agent {agent_name} in workspace {workspace_id}"
+                                )
+                                self._add_error_transcript_entry(
+                                    workspace, workspace_id, "self_message_loop_blocked",
+                                    f"Agent {agent_name} attempted self-message loop"
+                                )
+                                continue
+                            
+                            # Recipient validation already done in _validate_and_normalize_response
+                            # No need to check again here
+                            
                             # Display the message
                             if self.display:
                                 self.display.display_message(
                                     sender=agent_name,
-                                    recipient="USER",
-                                    content=agent_response["content"]
+                                    recipient=single_response["send_to"],
+                                    content=single_response["content"]
                                 )
                             
-                            await workspace.route_message(
+                            # Route message with deterministic ID
+                            message_id = await workspace.route_message(
                                 sender=agent_name,
-                                recipient="USER",
-                                content=agent_response["content"],
+                                recipient=single_response["send_to"],
+                                content=single_response["content"],
                                 manager=self.workspace_manager,
                             )
-                            cycle_had_output = True
-                            continue
-                        
-                        # Check for subteam request
-                        subteam_handled = await self._handle_subteam_request(
-                            workspace=workspace,
-                            workspace_id=workspace_id,
-                            agent_name=agent_name,
-                            response=agent_response,
-                            scheduler=scheduler,  # Only used for error cases
-                        )
-                        
-                        if subteam_handled:
-                            cycle_had_output = True
-                            continue
-                        
-                        # Check for infinite self-message loops
-                        if self._is_self_message_loop(
-                            agent, agent_response["send_to"], agent_name, agent_response["content"]
-                        ):
-                            logger.warning(
-                                f"Blocked self-message loop for agent {agent_name} in workspace {workspace_id}"
-                            )
-                            self._add_error_transcript_entry(
-                                workspace, workspace_id, "self_message_loop_blocked",
-                                f"Agent {agent_name} attempted self-message loop"
-                            )
-                            continue
-                        
-                        # Safety check: no events for nonexistent agents
-                        if agent_response["send_to"] not in workspace.agents and agent_response["send_to"] != "USER":
-                            agent_response["send_to"] = "USER"
-                        
-                        # Display the message
-                        if self.display:
-                            self.display.display_message(
-                                sender=agent_name,
-                                recipient=agent_response["send_to"],
-                                content=agent_response["content"]
-                            )
-                        
-                        # Route message with deterministic ID
-                        message_id = await workspace.route_message(
-                            sender=agent_name,
-                            recipient=agent_response["send_to"],
-                            content=agent_response["content"],
-                            manager=self.workspace_manager,
-                        )
-                        
-                        if message_id:
-                            # Schedule message event with message_id
-                            scheduler.schedule_event_dedup(AgentEvent(
-                                agent_name=agent_response["send_to"],
-                                trigger="message",
-                                payload={"message_id": message_id}
-                            ))
                             
-                            cycle_had_output = True
+                            if message_id:
+                                # Determine which workspace the recipient is in
+                                recipient_workspace_id = None
+                                if single_response["send_to"] in workspace.agents:
+                                    # Recipient is in current workspace
+                                    recipient_workspace_id = workspace_id
+                                elif self.workspace_manager:
+                                    # Check child workspaces
+                                    for child_id in workspace.children:
+                                        try:
+                                            child_workspace = self.workspace_manager.get_workspace(child_id)
+                                            if single_response["send_to"] in child_workspace.agents:
+                                                recipient_workspace_id = child_id
+                                                break
+                                        except:
+                                            pass
+                                    
+                                    # Check parent workspace
+                                    if not recipient_workspace_id and workspace.parent_id:
+                                        try:
+                                            parent_workspace = self.workspace_manager.get_workspace(workspace.parent_id)
+                                            if single_response["send_to"] in parent_workspace.agents:
+                                                recipient_workspace_id = workspace.parent_id
+                                        except:
+                                            pass
+                                
+                                # Schedule event in the appropriate workspace's scheduler
+                                if recipient_workspace_id:
+                                    # Get or create scheduler for the target workspace
+                                    if recipient_workspace_id not in self._workspace_schedulers:
+                                        self._workspace_schedulers[recipient_workspace_id] = EventScheduler()
+                                    
+                                    target_scheduler = self._workspace_schedulers[recipient_workspace_id]
+                                    target_scheduler.schedule_event_dedup(AgentEvent(
+                                        agent_name=single_response["send_to"],
+                                        trigger="message",
+                                        payload={"message_id": message_id}
+                                    ))
+                                    
+                                    # For cross-workspace messages, schedule the target workspace for execution
+                                    if recipient_workspace_id != workspace_id:
+                                        self.schedule_workspace(recipient_workspace_id)
+                                        logger.debug(
+                                            f"Cross-workspace message: {agent_name} → {single_response['send_to']} "
+                                            f"(workspace {workspace_id} → {recipient_workspace_id}), "
+                                            f"scheduled target workspace for execution"
+                                        )
+                                
+                                cycle_had_output = True
+                        
+                        # Increment agent turns counter AFTER processing all broadcast responses
+                        agent_turns += 1
+                        
+                        # Check max_agent_turns limit
+                        if self.max_agent_turns is not None and agent_turns >= self.max_agent_turns:
+                            if self.display:
+                                self.display.display_error(
+                                    "max_agent_turns_exceeded",
+                                    f"Stopped after {agent_turns} agent responses (task incomplete)"
+                                )
+                            logger.info(f"Max agent turns ({self.max_agent_turns}) reached, stopping workspace")
+                            task_complete = True
+                            # Clear remaining events
+                            while scheduler.has_pending_events():
+                                scheduler.pop_next_event()
+                            break
                     
                     except Exception as e:
                         logger.error(f"Error executing agent {agent_name} in workspace {workspace_id}: {e}")
@@ -567,21 +683,23 @@ class WorkspaceExecutionEngine:
         result: dict[str, str],
         agent_name: str,
         workspace: Workspace,
-    ) -> Optional[dict[str, str]]:
+    ) -> Optional[dict[str, str] | list[dict[str, str]]]:
         """
-        Validate and normalize agent response with strict rules.
+        Validate and normalize agent response with broadcast support.
         
-        Agent.process() guarantees a dict with "send_to" and "content", so this method
-        only validates fields, handles nonexistent send_to by redirecting to USER,
-        truncates overly long content, and ignores empty content.
+        Supports both single and broadcast responses:
+        - Single: {"send_to": "Agent", "content": "..."}
+        - Broadcast: {"send_to": ["Agent1", "Agent2", "Agent3"], "content": "..."}
         
         Args:
-            result: dict[str, str] from Agent.process() with "send_to" and "content"
+            result: dict from Agent.process() with "send_to" (str or list) and "content"
             agent_name: Name of the agent for logging
             workspace: Workspace instance for agent validation
             
         Returns:
-            dict[str, str] with "send_to" and "content" if valid, None if invalid
+            dict[str, str] if single recipient
+            list[dict[str, str]] if broadcast to multiple recipients
+            None if invalid
         """
         # Validate required fields exist
         if "send_to" not in result or "content" not in result:
@@ -591,12 +709,44 @@ class WorkspaceExecutionEngine:
         send_to = result["send_to"]
         content = result.get("content", "")
         
-        # Handle nonexistent send_to by redirecting to USER
-        if send_to not in workspace.agents and send_to != "USER":
-            logger.warning(
-                f"Agent {agent_name} sent to nonexistent agent '{send_to}', converting to USER"
-            )
-            send_to = "USER"
+        # BROADCAST SUPPORT: Handle list of recipients
+        if isinstance(send_to, list):
+            if not send_to:
+                logger.warning(f"Agent {agent_name} returned empty recipient list")
+                return None
+            
+            logger.info(f"Agent {agent_name} broadcasting to {len(send_to)} recipients: {send_to}")
+            
+            # Validate and normalize each recipient
+            normalized_responses = []
+            for recipient in send_to:
+                if not isinstance(recipient, str):
+                    logger.warning(f"Agent {agent_name} returned non-string recipient in list: {recipient}")
+                    continue
+                
+                # Validate this recipient exists (using same logic as single recipient)
+                validated_recipient = self._validate_single_recipient(recipient.strip(), workspace, agent_name)
+                if validated_recipient:
+                    normalized_responses.append({
+                        "send_to": validated_recipient,
+                        "content": content
+                    })
+            
+            if not normalized_responses:
+                logger.warning(f"Agent {agent_name} broadcast had no valid recipients")
+                return None
+            
+            return normalized_responses
+        
+        # SINGLE RECIPIENT: Use validation helper
+        if not isinstance(send_to, str):
+            logger.warning(f"Agent {agent_name} returned non-string send_to: {type(send_to)}")
+            return None
+        
+        # Validate recipient
+        send_to = self._validate_single_recipient(send_to.strip(), workspace, agent_name)
+        if not send_to:
+            return None
         
         # Truncate content if too long
         if len(content) > 10000:
@@ -610,6 +760,57 @@ class WorkspaceExecutionEngine:
         
         return {"send_to": send_to, "content": content}
     
+    def _validate_single_recipient(
+        self,
+        send_to: str,
+        workspace: Workspace,
+        agent_name: str
+    ) -> Optional[str]:
+        """
+        Validate a single recipient exists in workspace hierarchy.
+        
+        Returns the validated recipient name, or "USER" if not found, or None if invalid.
+        """
+        if not send_to:
+            return None
+        
+        # Handle nonexistent send_to by checking workspace hierarchy
+        recipient_exists = False
+        if send_to == "USER":
+            recipient_exists = True
+        elif send_to in workspace.agents:
+            recipient_exists = True
+        elif self.workspace_manager:
+            # Check child workspaces
+            for child_id in workspace.children:
+                try:
+                    child_workspace = self.workspace_manager.get_workspace(child_id)
+                    if send_to in child_workspace.agents:
+                        recipient_exists = True
+                        logger.debug(f"Found recipient '{send_to}' in child workspace {child_id}")
+                        break
+                except Exception as e:
+                    logger.debug(f"Error checking child workspace {child_id}: {e}")
+            
+            # Check parent workspace if not found in children
+            if not recipient_exists and workspace.parent_id:
+                try:
+                    parent_workspace = self.workspace_manager.get_workspace(workspace.parent_id)
+                    if send_to in parent_workspace.agents:
+                        recipient_exists = True
+                        logger.debug(f"Found recipient '{send_to}' in parent workspace {workspace.parent_id}")
+                except Exception as e:
+                    logger.debug(f"Error checking parent workspace {workspace.parent_id}: {e}")
+        
+        if not recipient_exists:
+            logger.warning(
+                f"Agent {agent_name} sent to nonexistent agent '{send_to}', converting to USER. "
+                f"Checked: current workspace {workspace.workspace_id}, {len(workspace.children)} children, parent: {workspace.parent_id}"
+            )
+            return "USER"
+        
+        return send_to
+    
     def _is_self_message_loop(
         self,
         agent: Any,
