@@ -27,37 +27,100 @@ logger = logging.getLogger(__name__)
 
 class AgentRuntimeRegistry:
     """
-    Prototype registry for agent roles.
+    Universal registry for agent roles - supports both local and remote A2A agents.
+    
+    This registry stores:
+    - Local agent prototypes (templates) built with Synqed
+    - Remote A2A agent endpoints (any agent implementing A2A protocol)
     
-    This registry stores agent prototypes (templates) for each role.
     When creating workspaces, use create_instance() to get fresh, independent
-    Agent instances cloned from these prototypes.
+    Agent instances cloned from local prototypes, or RemoteA2AAgent instances
+    for remote endpoints.
     
     Each workspace gets its own Agent instances - no Agent instance is shared
     between workspaces. This prevents shared mutable state issues.
     
-    In a real implementation, this would connect to actual agent runtimes.
-    For now, it's a prototype registry that clones agents per workspace.
+    UNIVERSAL SUBSTRATE:
+    Any agent that implements the A2A protocol can be registered here.
+    The agent just needs to provide:
+    - Endpoint URL
+    - AgentCard schema (/.well-known/agent-card.json)
+    - Authentication details (optional)
+    
+    Synqed will handle routing messages to it.
     """
     
-    _instances: dict[str, Agent] = {}  # role -> prototype Agent
+    _local_prototypes: dict[str, Agent] = {}  # role -> local prototype Agent
+    _remote_agents: dict[str, Any] = {}  # role -> RemoteA2AAgent config
     
     @classmethod
     def register(cls, role: str, agent: Agent) -> None:
         """
-        Register an agent prototype for a role.
+        Register a local agent prototype for a role.
         
         Args:
             role: Agent role name (e.g., "Writer", "Editor")
-            agent: Agent prototype instance
+            agent: Local Agent prototype instance
         """
-        cls._instances[role] = agent
-        logger.debug(f"Registered agent prototype for role '{role}'")
+        cls._local_prototypes[role] = agent
+        logger.debug(f"Registered local agent prototype for role '{role}'")
+    
+    @classmethod
+    def register_remote(
+        cls,
+        role: str,
+        url: str,
+        name: Optional[str] = None,
+        auth_token: Optional[str] = None,
+        transport: str = "JSONRPC",
+        description: Optional[str] = None
+    ) -> None:
+        """
+        Register a remote A2A-compliant agent.
+        
+        This allows ANY agent from ANY ecosystem to participate in Synqed workspaces,
+        as long as it implements the A2A protocol.
+        
+        The agent just needs to say:
+        - Here is my endpoint URL
+        - Here is my schema (A2A AgentCard at /.well-known/agent-card.json)
+        - Here is my auth (optional)
+        
+        And Synqed will route messages to it.
+        
+        Args:
+            role: Agent role name (e.g., "ExternalSpecialist")
+            url: A2A endpoint URL (e.g., "https://my-agent.example.com")
+            name: Optional agent name (fetched from AgentCard if not provided)
+            auth_token: Optional authentication token
+            transport: Transport protocol (JSONRPC, GRPC, or HTTP+JSON)
+            description: Optional description (fetched from AgentCard if not provided)
+            
+        Example:
+            ```python
+            # Register a remote A2A agent from any source
+            AgentRuntimeRegistry.register_remote(
+                role="CodeReviewAgent",
+                url="https://code-review-agent.example.com",
+                auth_token="secret-key"
+            )
+            
+            # Now it can be used in any workspace
+            ```
+        """
+        cls._remote_agents[role] = {
+            "url": url,
+            "name": name,
+            "auth_token": auth_token,
+            "transport": transport,
+            "description": description
+        }
+        logger.info(f"Registered remote A2A agent for role '{role}' at {url}")
     
     @classmethod
     def get(cls, role: str) -> Optional[Agent]:
         """
-        Get the agent prototype for a role (for introspection only).
+        Get the local agent prototype for a role (for introspection only).
         
         NOTE: This returns the prototype, not a workspace-specific instance.
         Use create_instance() whenever attaching an Agent to a Workspace.
@@ -68,41 +131,63 @@ class AgentRuntimeRegistry:
         Returns:
             Agent prototype if found, None otherwise
         """
-        return cls._instances.get(role)
+        return cls._local_prototypes.get(role)
     
     @classmethod
-    def create_instance(cls, role: str, workspace_id: Optional[str] = None) -> Optional[Agent]:
+    def create_instance(cls, role: str, workspace_id: Optional[str] = None) -> Optional[Any]:
         """
-        Create a fresh Agent instance cloned from the prototype.
+        Create a fresh agent instance for a workspace.
         
-        This method deep copies the prototype so each workspace gets its own
-        independent Agent instance. This prevents shared mutable state.
+        This method:
+        - For local agents: deep copies the prototype
+        - For remote A2A agents: creates a new RemoteA2AAgent client
+        
+        Each workspace gets its own independent instance.
         
         Args:
             role: Agent role name
             workspace_id: Optional workspace ID for debugging annotation
             
         Returns:
-            Cloned Agent instance if prototype found, None otherwise
+            Agent instance (local Agent or RemoteA2AAgent) if found, None otherwise
         """
-        prototype = cls._instances.get(role)
-        if prototype is None:
-            return None
+        # Check for local agent prototype
+        local_prototype = cls._local_prototypes.get(role)
+        if local_prototype is not None:
+            # Deep copy the prototype
+            cloned_agent = copy.deepcopy(local_prototype)
+            
+            # Annotate with workspace_id for debugging
+            if workspace_id is not None:
+                cloned_agent.workspace_id = workspace_id  # type: ignore
+            
+            logger.debug(f"Created local agent instance for role '{role}' (workspace: {workspace_id})")
+            return cloned_agent
         
-        # Deep copy the prototype
-        cloned_agent = copy.deepcopy(prototype)
+        # Check for remote A2A agent
+        remote_config = cls._remote_agents.get(role)
+        if remote_config is not None:
+            # Import here to avoid circular dependency
+            from synqed.a2a_remote_agent import RemoteA2AAgent
+            
+            # Create new RemoteA2AAgent instance
+            remote_agent = RemoteA2AAgent(
+                url=remote_config["url"],
+                name=remote_config.get("name", role),
+                auth_token=remote_config.get("auth_token"),
+                transport=remote_config.get("transport", "JSONRPC"),
+                description=remote_config.get("description")
+            )
+            
+            logger.debug(f"Created remote A2A agent instance for role '{role}' (workspace: {workspace_id})")
+            return remote_agent
         
-        # Annotate with workspace_id for debugging (does not change external semantics)
-        if workspace_id is not None:
-            cloned_agent.workspace_id = workspace_id  # type: ignore
-        
-        logger.debug(f"Created agent instance for role '{role}' (workspace: {workspace_id})")
-        return cloned_agent
+        return None
     
     @classmethod
     def has(cls, role: str) -> bool:
         """
-        Check if a role is registered.
+        Check if a role is registered (local or remote).
         
         Args:
             role: Agent role name
@@ -110,22 +195,23 @@ class AgentRuntimeRegistry:
         Returns:
             True if role is registered
         """
-        return role in cls._instances
+        return role in cls._local_prototypes or role in cls._remote_agents
     
     @classmethod
     def list_roles(cls) -> list[str]:
         """
-        List all registered roles.
+        List all registered roles (local and remote).
         
         Returns:
             List of role names
         """
-        return list(cls._instances.keys())
+        return list(set(cls._local_prototypes.keys()) | set(cls._remote_agents.keys()))
     
     @classmethod
     def clear(cls) -> None:
-        """Clear all registered agents."""
-        cls._instances.clear()
+        """Clear all registered agents (local and remote)."""
+        cls._local_prototypes.clear()
+        cls._remote_agents.clear()
 
 
 class Workspace(BaseModel):
@@ -326,17 +412,22 @@ class Workspace(BaseModel):
         for agent_name in self.agents.keys():
             await self.route_message(sender, agent_name, content, manager)
     
-    def add_agent(self, agent: Agent) -> None:
+    def add_agent(self, agent: Any) -> None:
         """
         Add an agent to this workspace.
         
+        Supports both:
+        - Local agents (built with Synqed)
+        - Remote A2A agents (any agent implementing A2A protocol)
+        
         Args:
-            agent: Agent instance to add
+            agent: Agent instance (local Agent or RemoteA2AAgent)
         """
         self.agents[agent.name] = agent
         
-        # Register agent with router for local routing
-        self.router.register_local_agent(agent.name, agent)
+        # Register agent with router
+        # Router will handle both local and remote agents
+        self.router.register_agent(agent.name, agent)
         
         logger.debug(f"Added agent '{agent.name}' to workspace {self.workspace_id}")
     
@@ -387,6 +478,147 @@ class Workspace(BaseModel):
         }
         self.router.add_transcript_entry(transcript_entry)
     
+    def display_transcript(
+        self,
+        include_system_messages: bool = False,
+        parse_json_content: bool = True,
+        title: Optional[str] = None
+    ) -> None:
+        """
+        Display the complete conversation transcript in a readable format.
+        
+        This utility method displays the transcript with automatic JSON parsing
+        and formatting, making it easy to review agent conversations.
+        
+        Args:
+            include_system_messages: Whether to include system messages like [startup]
+            parse_json_content: Whether to parse JSON content and extract "content" field
+            title: Optional title to display above the transcript
+        """
+        import json
+        
+        if title:
+            print("\n" + "="*80)
+            print(f"  {title}")
+            print("="*80 + "\n")
+        
+        transcript = self.router.get_transcript()
+        message_num = 0
+        
+        for entry in transcript:
+            sender = entry.get("from", "?")
+            recipient = entry.get("to", "?")
+            content = entry.get("content", "")
+            
+            # Skip system messages unless requested
+            if not include_system_messages:
+                if content == "[startup]" or content.startswith("[subteam_result]"):
+                    continue
+            
+            message_num += 1
+            
+            print(f"[Message {message_num}] {sender} → {recipient}")
+            print("-" * 80)
+            
+            # Parse and display content
+            display_content = content
+            if parse_json_content:
+                try:
+                    parsed = json.loads(content)
+                    if isinstance(parsed, dict) and "send_to" in parsed and "content" in parsed:
+                        # It's a structured message, show the actual content
+                        display_content = parsed["content"]
+                except (json.JSONDecodeError, TypeError):
+                    # Not JSON, show as-is
+                    pass
+            
+            print(display_content)
+            print()
+        
+        if message_num == 0:
+            print("(No messages in transcript)")
+            print()
+    
+    def get_completion_status(self) -> dict:
+        """
+        Analyze the transcript to determine task completion status.
+        
+        This utility method checks if:
+        - Any message was successfully sent to USER (task completed)
+        - Any attempted messages to USER (malformed/truncated)
+        
+        Returns:
+            Dictionary with completion status information:
+            {
+                "completed": bool,  # True if message successfully sent to USER
+                "attempted": int,   # Number of attempted USER messages (malformed)
+                "total_messages": int,  # Total non-system messages
+                "status_message": str  # Human-readable status
+            }
+        """
+        transcript = self.router.get_transcript()
+        
+        # Check if task completed (any message to USER)
+        task_completed = any(
+            entry.get("to") == "USER" 
+            for entry in transcript
+            if entry.get("content", "") not in ["[startup]", ""] 
+            and not entry.get("content", "").startswith("[subteam_result]")
+        )
+        
+        # Count malformed attempts to send to USER
+        attempted_user_messages = 0
+        for entry in transcript:
+            content = entry.get("content", "")
+            if entry.get("to") != "USER":  # Only count non-routed attempts
+                if '"send_to": "USER"' in content or '"send_to":"USER"' in content:
+                    attempted_user_messages += 1
+        
+        # Count total non-system messages
+        total_messages = sum(
+            1 for entry in transcript
+            if entry.get("content", "") not in ["[startup]", ""]
+            and not entry.get("content", "").startswith("[subteam_result]")
+        )
+        
+        # Build status message
+        if task_completed:
+            status_message = "✅ Task completed successfully (message sent to USER)"
+        elif attempted_user_messages > 0:
+            status_message = (
+                f"⚠️  Task incomplete ({attempted_user_messages} attempt(s) to send to USER, "
+                f"but JSON was malformed/truncated)\n"
+                f"    Suggestion: Increase max_tokens to allow complete responses"
+            )
+        else:
+            status_message = "⚠️  Task incomplete (no message sent to USER)"
+        
+        return {
+            "completed": task_completed,
+            "attempted": attempted_user_messages,
+            "total_messages": total_messages,
+            "status_message": status_message
+        }
+    
+    def print_summary(self) -> None:
+        """
+        Print a summary of the workspace execution.
+        
+        This displays:
+        - Total message count
+        - Completion status
+        - Any relevant warnings or suggestions
+        """
+        status = self.get_completion_status()
+        
+        print("="*80)
+        print("  SUMMARY")
+        print("="*80)
+        print(f"Workspace: {self.workspace_id}")
+        print(f"Total messages: {status['total_messages']}")
+        print(f"Status: {status['status_message']}")
+        print()
+    
     def __repr__(self) -> str:
         """String representation."""
         return (
@@ -541,8 +773,9 @@ class WorkspaceManager:
         
         # Initialize agent memory with workspace_id and register agents
         for agent_role, agent in agents.items():
-            # Initialize memory with workspace_id
-            agent.memory.workspace_id = workspace_id
+            # Initialize memory with workspace_id (only for local agents)
+            if hasattr(agent, 'memory'):
+                agent.memory.workspace_id = workspace_id
             workspace.add_agent(agent)
         
         # Register workspace
